
/*
 * Copyright (c) 2001-2002 Packet Design, LLC.
 * All rights reserved.
 * 
 * Subject to the following obligations and disclaimer of warranty,
 * use and redistribution of this software, in source or object code
 * forms, with or without modifications are expressly permitted by
 * Packet Design; provided, however, that:
 * 
 *    (i)  Any and all reproductions of the source or object code
 *         must include the copyright notice above and the following
 *         disclaimer of warranties; and
 *    (ii) No rights are granted, in any manner or form, to use
 *         Packet Design trademarks, including the mark "PACKET DESIGN"
 *         on advertising, endorsements, or otherwise except as such
 *         appears in the above copyright notice or in the software.
 * 
 * THIS SOFTWARE IS BEING PROVIDED BY PACKET DESIGN "AS IS", AND
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, PACKET DESIGN MAKES NO
 * REPRESENTATIONS OR WARRANTIES, EXPRESS OR IMPLIED, REGARDING
 * THIS SOFTWARE, INCLUDING WITHOUT LIMITATION, ANY AND ALL IMPLIED
 * WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE,
 * OR NON-INFRINGEMENT.  PACKET DESIGN DOES NOT WARRANT, GUARANTEE,
 * OR MAKE ANY REPRESENTATIONS REGARDING THE USE OF, OR THE RESULTS
 * OF THE USE OF THIS SOFTWARE IN TERMS OF ITS CORRECTNESS, ACCURACY,
 * RELIABILITY OR OTHERWISE.  IN NO EVENT SHALL PACKET DESIGN BE
 * LIABLE FOR ANY DAMAGES RESULTING FROM OR ARISING OUT OF ANY USE
 * OF THIS SOFTWARE, INCLUDING WITHOUT LIMITATION, ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, PUNITIVE, OR CONSEQUENTIAL
 * DAMAGES, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES, LOSS OF
 * USE, DATA OR PROFITS, HOWEVER CAUSED AND UNDER ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF PACKET DESIGN IS ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 *
 * Author: Archie Cobbs <archie@freebsd.org>
 */

#ifndef _PDEL_UTIL_KERNELGLUE_H_
#define _PDEL_UTIL_KERNELGLUE_H_

#ifndef _KERNEL
#error This #include file is only for use within a kernel compilation
#endif

#ifdef _PDEL_UTIL_TYPED_MEM_H_
#error kernelglue.h must be included before typed_mem.h
#endif

/*
 * Undefine kernel MALLOC() and FREE() macros
 */
#undef MALLOC
#undef FREE

/*
 * Declare memory type for typed memory chunks
 */
MALLOC_DECLARE(M_LIBPDEL);

/*
 * Kernel glue for 'errno'
 */
extern int errno;

/*
 * Kernel glue for misc user-land macros
 */
#define MAX(x,y)	max(x,y)
#define MIN(x,y)	min(x,y)
#define assert(x) 	KASSERT((x), \
			    ("%s: assert failed: %s", __FUNCTION__, #x))

/*
 * Kernel glue for misc user-land functions
 */
extern char		*strchr(const char *s, int c);
extern char		*strdup(const char *s);
extern int		asprintf(char **ret, const char *format, ...);
extern int		vasprintf(char **ret, const char *format, va_list ap);
extern int		strcasecmp(const char *s1, const char *s2);
extern size_t		strlcpy(char *dst, const char *src, size_t siz);
extern char		*strerror(int errnum);
extern time_t		time(time_t *);
extern void		*memmove(void *dst, const void *src, size_t len);
extern void		*kern_malloc(size_t size);
extern void		*kern_realloc(void *mem, size_t size);
extern void		kern_free(void *mem);

#endif	/* _PDEL_UTIL_KERNELGLUE_H_ */

